%global iptables_init_scriptpath %{_libexecdir}/iptables
%global legacy_action_scriptpath %{_libexecdir}/initscripts/legacy-actions
%global iptables_prefix %{_sbindir}/iptables
%global ip6tables_prefix %{_sbindir}/ip6tables

Summary: The userspace command line program of netfilter for packets filtering
Name: iptables
Version: 1.8.9
Release: 1%{?dist}
# iptables: GNU GPLv2
# pf.os: ISC license
# iptables-apply: Artistic Licence 2.0
License: GPLv2 and Artistic Licence 2.0 and ISC
URL: https://www.netfilter.org/projects/iptables
Source0: %{url}/files/%{name}-%{version}.tar.xz
Source1: iptables.init
Source2: iptables-config
Source3: iptables.service
Source4: sysconfig_iptables
Source5: sysconfig_ip6tables
Source6: arptables-helper
Source7: arptables.service
Source8: ebtables.service
Source9: ebtables-config
Source10: ebtables.systemd

# Patches are converted to apply with '-p1'
# no patches now


BuildRequires: gcc autoconf automake make libtool
BuildRequires: pkgconfig(libnetfilter_conntrack) pkgconfig(libnfnetlink)
BuildRequires: pkgconfig(libmnl) >= 1.0 pkgconfig(libnftnl) >= 1.1.5 bison flex libpcap-devel
BuildRequires: kernel-headers systemd

%description 
Iptables is the userspace command line program used to configure the Linux 2.4.x 
and later packet filtering ruleset. It is targeted towards system administrators.
Since Network Address Translation is also configured from the packet filter ruleset,
iptables is used for this, too.The iptables package also includes ip6tables. 
Ip6tables is used for configuring the IPv6 packet filter.


%package legacy
Summary:  iptables using old getsockopt/setsockopt-based kernel api
Requires: %{name}-legacy-libs = %{version}-%{release} %{name}-libs = %{version}-%{release}
Requires(post): %{_sbindir}/update-alternatives
Requires(postun): %{_sbindir}/update-alternatives
Provides: iptables iptables-services

%description legacy
iptables-legacy are the original versions of iptables that use old getsockopt/
setsockopt-based kernel interface.  This kernel interface has some limitations, 
therefore iptables can also be used with the newer nf_tables based API.If you 
need to set up firewalls and/or IP masquerading, you should not install this 
package but iptables-nft instead.


%package libs
Summary: libxtables and associated shared object files

%description libs
Libxtables provides unified access to iptables extensions in userspace. Data
and logic for those is kept in per-extension shared object files.


%package legacy-libs
Summary: iptables legacy libraries

%description legacy-libs
This package provides development headers files and static libraries for libip4tc
and libip6tc.


%package devel
Summary: Development package for iptables
Requires: %{name}-libs = %{version}-%{release} %{name}-legacy-devel = %{version}-%{release}
Requires: pkgconfig

%description devel
libxtables development headers and libraries,support pkgconfig


%package legacy-devel
Summary: Development package for legacy iptables
Requires: %{name}-legacy-libs = %{version}-%{release}
Requires: pkgconfig

%description legacy-devel
This package contains legacy iptables development headers and libraries.
The libip*tc libraries are marked as not public by upstream. The interface is 
not stable and may change with every new version.


%package utils
Summary: iptables utilities include nfnl_osf,nfbpf_compile and iptables-apply
Requires: %{name} = %{version}-%{release}

%description utils
This package provides nfnl_osf with the pf.os database and nfbpf_compile,
a bytecode generator for use with xt_bpf. And also provides iptables-apply,
a safer way to update iptables remotely.


%package nft
Summary: nftables compatibility for iptables, arptables and ebtables
Requires: %{name}-libs = %{version}-%{release}
Requires(post): %{_sbindir}/update-alternatives  %{_bindir}/readlink
Requires(postun): %{_sbindir}/update-alternatives
Provides: arptables-helper arptables ebtables iptables

%description nft
nftables compatibility for iptables, arptables and ebtables.


%prep
%autosetup


%build
./autogen.sh
CFLAGS=${CFLAGS/-Werror=format-security/};export CFLAGS;
%configure --enable-devel --enable-bpf-compiler --with-kernel=/usr --with-kbuild=/usr --with-ksource=/usr
# disable rpath to avoid hardcode specific library paths and also for safety 
sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
sed -i 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' libtool
%make_build


%install
%make_install
rm -f %{buildroot}%{_libdir}/*.la
# install init scripts
install -d -m 755 %{buildroot}%{iptables_init_scriptpath}
install -m 755 %{SOURCE1} %{buildroot}%{iptables_init_scriptpath}/iptables.init
sed -e 's|iptables|ip6tables|g' -e 's|IPTABLES|IP6TABLES|g' %{SOURCE1} > ip6tables.init
install -m 755 ip6tables.init %{buildroot}%{iptables_init_scriptpath}/ip6tables.init
install -p -D -m 755 %{SOURCE6} %{buildroot}%{_libexecdir}/
install -m 755 %{SOURCE10} %{buildroot}%{_libexecdir}/ebtables

# install configure files
install -d -m 755 %{buildroot}%{_sysconfdir}/sysconfig
install -m 600 %{SOURCE2} %{buildroot}%{_sysconfdir}/sysconfig/iptables-config
sed -e 's|iptables|ip6tables|g' -e 's|IPTABLES|IP6TABLES|g' %{SOURCE2} > ip6tables-config
install -m 600 ip6tables-config %{buildroot}%{_sysconfdir}/sysconfig/ip6tables-config
install -m 600 %{SOURCE4} %{buildroot}%{_sysconfdir}/sysconfig/iptables
install -m 600 %{SOURCE5} %{buildroot}%{_sysconfdir}/sysconfig/ip6tables
install -m 600 %{SOURCE9} %{buildroot}%{_sysconfdir}/sysconfig/ebtables-config
echo '# Configure prior to use' > %{buildroot}%{_sysconfdir}/sysconfig/arptables-config

# install systemd service files
install -d -m 755 %{buildroot}/%{_unitdir}
sed -e 's|iptables|ip6tables|g' -e 's|IPv4|IPv6|g' -e 's|/usr/libexec/ip6tables|/usr/libexec/iptables|g' %{SOURCE3} > ip6tables.service
install -m 644 %{SOURCE3} %{buildroot}/%{_unitdir}
install -m 644 ip6tables.service %{buildroot}/%{_unitdir}
install -m 644 %{SOURCE7} %{buildroot}%{_unitdir}
install -m 644 %{SOURCE8} %{buildroot}%{_unitdir}
# can't provide these actions in systemd,so install legacy actions for service command at this place
install -d %{buildroot}/%{legacy_action_scriptpath}/iptables
install -d %{buildroot}/%{legacy_action_scriptpath}/ip6tables
for action in panic save;
do
cat << EOF > %{buildroot}/%{legacy_action_scriptpath}/iptables/$action
#!/bin/bash
exec %{iptables_init_scriptpath}/iptables.init $action
EOF
  chmod 755 %{buildroot}/%{legacy_action_scriptpath}/iptables/$action

cat << EOF > %{buildroot}/%{legacy_action_scriptpath}/ip6tables/$action
#!/bin/bash
exec %{iptables_init_scriptpath}/ip6tables.init $action
EOF
  chmod 755 %{buildroot}/%{legacy_action_scriptpath}/ip6tables/$action
done

# install manpages
pushd %{_mandir}/man8/
for suffix in ".8.gz" "-save.8.gz" "-restore.8.gz";
do
    ln -sf arptables-nft$suffix %{buildroot}%{_mandir}/man8/arptables$suffix
done
ln -sf ebtables-nft.8.gz %{buildroot}%{_mandir}/man8/ebtables.8.gz
popd

# install executable files
pushd %{_sbindir}/
#link bin files of ebtables and arptables
for suffix in "" "-save" "-restore";
do
    ln -sf ebtables-nft$suffix %{buildroot}%{_sbindir}/ebtables$suffix
	ln -sf arptables-nft$suffix %{buildroot}%{_sbindir}/arptables$suffix
done
popd

rm -f %{buildroot}%{_sysconfdir}/xtables.conf
rm -f %{buildroot}%{_bindir}/iptables-xml
ln -s ../sbin/xtables-legacy-multi %{buildroot}%{_bindir}/iptables-xml

%post -p /sbin/ldconfig


%postun -p /sbin/ldconfig


%post legacy
%systemd_post iptables.service ip6tables.service
# use update-alternatives to install iptables for version control,also it provides a easy way
# to switch between iptables-legacy and iptables-nft
update-alternatives --install \
	%{iptables_prefix} iptables %{iptables_prefix}-legacy 10 \
	--slave %{ip6tables_prefix} ip6tables %{ip6tables_prefix}-legacy \
	--slave %{iptables_prefix}-restore iptables-restore %{iptables_prefix}-legacy-restore \
	--slave %{iptables_prefix}-save iptables-save %{iptables_prefix}-legacy-save \
	--slave %{ip6tables_prefix}-restore ip6tables-restore %{ip6tables_prefix}-legacy-restore \
	--slave %{ip6tables_prefix}-save ip6tables-save %{ip6tables_prefix}-legacy-save


%preun legacy
%systemd_preun iptables.service ip6tables.service


%postun legacy
%systemd_postun iptables.service ip6tables.service
# delete iptables alternatives after uninstall
if [ $1 -eq 0 ]; then
	update-alternatives --remove \
		iptables %{_sbindir}/iptables-legacy
fi


%post nft
%systemd_post arptables.service ebtables.service
# use update-alternatives to install iptables for version control,also it provides a easy way
# to switch between iptables-legacy and iptables-nft
update-alternatives --install \
	%{iptables_prefix} iptables %{iptables_prefix}-nft 10 \
	--slave %{ip6tables_prefix} ip6tables %{ip6tables_prefix}-nft \
	--slave %{iptables_prefix}-restore iptables-restore %{iptables_prefix}-nft-restore \
	--slave %{iptables_prefix}-save iptables-save %{iptables_prefix}-nft-save \
	--slave %{ip6tables_prefix}-restore ip6tables-restore %{ip6tables_prefix}-nft-restore \
	--slave %{ip6tables_prefix}-save ip6tables-save %{ip6tables_prefix}-nft-save


%preun nft
%systemd_preun arptables.service ebtables.service


%postun nft
%systemd_postun arptables.service
%systemd_postun_with_restart ebtables.service
if [ $1 -eq 0 ]; then
	update-alternatives --remove \
		iptables %{_sbindir}/iptables-nft
fi


%files legacy
%license COPYING
%{_mandir}/man1/iptables-xml.1.gz
%{_mandir}/man8/xtables-legacy.8.gz
%{_mandir}/man8/ip{,6}tables.8.gz
%{_mandir}/man8/ip{,6}tables-extensions.8.gz
%{_mandir}/man8/ip{,6}tables-save.8.gz
%{_mandir}/man8/ip{,6}tables-restore.8.gz
%dir %{iptables_init_scriptpath}
%{iptables_init_scriptpath}/*
%config(noreplace) %{_sysconfdir}/sysconfig/ip{,6}tables
%config(noreplace) %{_sysconfdir}/sysconfig/ip{,6}tables-config
%config(noreplace) %{_sysconfdir}/ethertypes
%{_unitdir}/ip{,6}tables.service
%dir %{legacy_action_scriptpath}/ip{,6}tables
%{legacy_action_scriptpath}/ip{,6}tables/*
%{_bindir}/iptables-xml
%{_datadir}/xtables/iptables.xslt
%{_sbindir}/ip{,6}tables-legacy*
%{_sbindir}/xtables-legacy-multi
%ghost %{_sbindir}/ip{,6}tables-save
%ghost %{_sbindir}/ip{,6}tables-restore


%files libs
%{_libdir}/libxtables.so*
%dir %{_libdir}/xtables
%{_libdir}/xtables/libip{,6}t*.so
%{_libdir}/xtables/libxt*.so


%files legacy-libs
%{_libdir}/libip{4,6}tc.so.*


%files devel
%{_includedir}/xtables{,-version}.h
%{_libdir}/libxtables.so.12*
%{_libdir}/pkgconfig/xtables.pc


%files legacy-devel
%dir %{_includedir}/libiptc
%{_includedir}/libiptc/*.h
%{_libdir}/libip{4,6}tc.so
%{_libdir}/pkgconfig/libip{,4,6}tc.pc


%files utils
%{_mandir}/man8/nfnl_osf.8.gz
%{_mandir}/man8/nfbpf_compile.8.gz
%{_mandir}/man8/ip{,6}tables-apply.8.gz
%dir %{_datadir}/xtables
%{_datadir}/xtables/pf.os
%{_sbindir}/nfnl_osf
%{_sbindir}/nfbpf_compile
%{_sbindir}/ip{,6}tables-apply


%files nft
%{_mandir}/man8/xtables-monitor.8.gz
%{_mandir}/man8/xtables-translate.8.gz
%{_mandir}/man8/xtables-nft.8.gz
%{_mandir}/man8/{arp,eb}tables*.8.gz
%{_mandir}/man8/ip{,6}tables-translate.8.gz
%{_mandir}/man8/ip{,6}tables-restore-translate.8.gz
%dir %{_libdir}/xtables
%{_libdir}/xtables/lib{arpt,ebt}*.so
%{_libexecdir}/arptables-helper
%{_libexecdir}/ebtables
%{_unitdir}/arptables.service
%{_unitdir}/ebtables.service
%config(noreplace) %{_sysconfdir}/sysconfig/arptables-config
%config(noreplace) %{_sysconfdir}/sysconfig/ebtables-config
%{_sbindir}/ip{,6}tables*
%{_sbindir}/{eb,arp}tables*
%{_sbindir}/xtables-nft-multi
%{_sbindir}/xtables-monitor


%changelog
* Mon Jan 15 2024 Miaojun Dong <zoedong@tencent.com> - 1.8.9-1
- Upgrade to version 1.8.9

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.8.8-4
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.8.8-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.8.8-2
- Rebuilt for OpenCloudOS Stream 23

* Thu May 19 2022 doriscchao <doriscchao@tencent.com> - 1.8.8-1
- initial build,Version 1.8.8, Release 1
